/******************************************************************************\
 *           ___        __                                                    *
 *          /\_ \    __/\ \                                                   *
 *          \//\ \  /\_\ \ \____    ___   _____   _____      __               *
 *            \ \ \ \/\ \ \ '__`\  /'___\/\ '__`\/\ '__`\  /'__`\             *
 *             \_\ \_\ \ \ \ \L\ \/\ \__/\ \ \L\ \ \ \L\ \/\ \L\.\_           *
 *             /\____\\ \_\ \_,__/\ \____\\ \ ,__/\ \ ,__/\ \__/.\_\          *
 *             \/____/ \/_/\/___/  \/____/ \ \ \/  \ \ \/  \/__/\/_/          *
 *                                          \ \_\   \ \_\                     *
 *                                           \/_/    \/_/                     *
 *                                                                            *
 * Copyright (C) 2011-2013                                                    *
 * Dominik Charousset <dominik.charousset@haw-hamburg.de>                     *
 *                                                                            *
 * This file is part of libcppa.                                              *
 * libcppa is free software: you can redistribute it and/or modify it under   *
 * the terms of the GNU Lesser General Public License as published by the     *
 * Free Software Foundation; either version 2.1 of the License,               *
 * or (at your option) any later version.                                     *
 *                                                                            *
 * libcppa is distributed in the hope that it will be useful,                 *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                       *
 * See the GNU Lesser General Public License for more details.                *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public License   *
 * along with libcppa. If not, see <http://www.gnu.org/licenses/>.            *
\******************************************************************************/


#ifndef CPPA_ANYTHING_HPP
#define CPPA_ANYTHING_HPP

#include <type_traits>

namespace cppa {

/**
 * @brief Acts as wildcard expression in patterns.
 */
struct anything { };

/**
 * @brief Compares two instances of {@link anything}.
 * @returns @p false
 * @relates anything
 */
inline bool operator==(const anything&, const anything&) { return true; }

/**
 * @brief Compares two instances of {@link anything}.
 * @returns @p false
 * @relates anything
 */
inline bool operator!=(const anything&, const anything&) { return false; }

/**
 * @brief Checks wheter @p T is {@link anything}.
 */
template<typename T>
struct is_anything {
    static constexpr bool value = std::is_same<T, anything>::value;
};


} // namespace cppa

#endif // CPPA_ANYTHING_HPP
